//---------------------------------------------------------------------------
#pragma hdrstop
//-------------------------------------------------------------------------
#include <math.h>
#include "BPlano3D.h"
//---------------------------------------------------------------------------
BPlano3D::BPlano3D(void)
{
    D = 0;
}
//---------------------------------------------------------------------------
BPlano3D::BPlano3D(double a, double b, double c, double d)
{
    normal.DefCoordenadas(a, b, c);
    D = d;
}
//---------------------------------------------------------------------------
BPlano3D::BPlano3D(const BVetor3D& n, const BPonto3D& pto)
{
    normal = n;
    D = - n.X() * pto.X() - n.Y() * pto.Y() - n.Z() * pto.Z();
}
//---------------------------------------------------------------------------
BPlano3D::BPlano3D(const BVetor3D& n, double d)
{
    normal = n;
    D = d;
}
//---------------------------------------------------------------------------
BPlano3D::BPlano3D(const BPonto3D& p1, const BPonto3D& p2, const BPonto3D& p3)
{
    BVetor3D v1, v2;

    v1 = p2 - p1;
    v2 = p3 - p1;
    Def(ProdutoVetorial(v1, v2), p1);
}
//---------------------------------------------------------------------------
BPlano3D::BPlano3D(const BPlano3D& plano)
{
    normal = plano.normal;
    D = plano.D;
}
//---------------------------------------------------------------------------
BPlano3D::~BPlano3D(void)
{

}
//---------------------------------------------------------------------------
BPlano3D& BPlano3D::operator = (const BPlano3D& plano)
{
    normal = plano.normal;
    D = plano.D;
    return *this;
}
//---------------------------------------------------------------------------
void BPlano3D::Def(double a, double b, double c, double d)
{
    normal.DefCoordenadas(a, b, c);
    D = d;
}
//---------------------------------------------------------------------------
void BPlano3D::Def(const BVetor3D& n, const BPonto3D& pto)
{
    normal = n;
    D = - n.X() * pto.X() - n.Y() * pto.Y() - n.Z() * pto.Z();
}
//---------------------------------------------------------------------------
void BPlano3D::Def(const BVetor3D& n, double d)
{
    normal = n;
    D = d;
}
//---------------------------------------------------------------------------
void BPlano3D::Def(const BPonto3D& p1, const BPonto3D& p2, const BPonto3D& p3)
{
    BVetor3D v1, v2;

    v1 = p2 - p1;
    v2 = p3 - p1;
    Def(ProdutoVetorial(v1, v2), p1);
}
//---------------------------------------------------------------------------
BPonto3D BPlano3D::PontoQualquer(void)
{
/*
      Esta funo retorna um ponto qualquer pertencente ao plano
*/
    BPonto3D pt;

    if(normal.Z() == 0)
    {
        if(normal.Y() == 0)
        {
            if(normal.X() == 0)
            {
                throw;
            }
            else
            {
                pt.DefCoordenadas(-D / normal.X(), 0, 0);
            }
        }
        else
        {
            pt.DefCoordenadas(0, -D / normal.Y(), 0);
        }
    }
    else
    {
        pt.DefCoordenadas(0, -D / normal.Y(), 0);
    }
    return pt;
}
//---------------------------------------------------------------------------
int BPlano3D::PosicaoDoPonto(const BPonto3D& pto)
{
/*
      Esta funo retorna:
      1 se o ponto estiver acima do plano
      0 se o ponto estiver na superfcie do plano
     -1 se o ponto estiver abaixo do plano

      Entende-se por "acima do plano", na direo do vetor normal.
*/
   BPonto3D pt = PontoQualquer();
   double m;
   double ZERO = 1.0e-7;

   m = normal * (pto - pt);
   if(fabs(m) < ZERO) return 0;
   if(m < 0) return -1;
   return 1;
}
//---------------------------------------------------------------------------

